#include <iostream>
#include <sstream>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_math.h>
#include "hdf5.h"

// macro definitions
#define VERSION "0.1 -- 11 November 2013"
#define NGEN 3 // number of genotypes, set to 3 for diploid inds. and bi-allelic loci

extern gsl_rng * r;
// function declarations
using namespace std;


// struct definitions

struct datacont{
  gsl_matrix ** gl; // genotype likelihoods; locus x ind x genotype; Hom Ref, Het, Hom Alt
  gsl_vector_int * pincl; // boolean, include individual in allele frequency estimate
  int nloci;
  int nind;
};



struct paramcont{
  gsl_matrix_int * g; // genotype; locus x ind
  gsl_vector * p; // reference allele frequency; locus
  gsl_matrix ** postg; // genotype posterior probability distn.; locus x ind x genotype; Hom Ref, Het, Hom Alt
  gsl_vector * aux; // aux vector for hdf5 write; locus
  double pprior; // parameter for beta prior on allele frequencies
};


struct hdf5cont{
  hid_t file;
  hid_t datatype;
  hid_t dataspaceLIG, dataspaceLM;

  hid_t datasetG, datasetP;
  hid_t locbuffer;

  /* dataset dimensions */ 
  hsize_t dimsLIG[3], dimsLM[3]; 
  herr_t status; /* not sure what this does, but it appears to be important?? */

  hsize_t sr3[3]; /* Start of hyperslab */
  hsize_t sr2[2];
  hsize_t cr3[3]; /* Block count */
  hsize_t cr2[2];
  hsize_t br3[3]; /* Block size */
  hsize_t br2[2];
};

// function definitions
void usage(char *);
void readData(string infile, datacont * data);
void createHdf5(hdf5cont * hdf5, datacont * data, int N);
void initParams(datacont * data, paramcont * params);
void mcmcUpdate(datacont * data, paramcont * params);
void updateP(datacont * data, paramcont * params);
void updateG(datacont * data, paramcont * params);
void writeStep(hdf5cont * hdf5, datacont * data, paramcont * params, int s);
void updatePostG(datacont * data, paramcont * params, int N);
void writePostG(hdf5cont * hdf5, datacont * data, paramcont * params);
