#!/usr/bin/perl

use warnings;
use strict;

# this program filters a vcf file based on overall sequence coverage, number of non-reference reads, number of alleles, and reverse orientation reads

# usage vcfFilter.pl infile.vcf
#
# change the marked variables below to adjust settings
#

#### stringency variables, edits as desired
my $minCoverage = 1500; # minimum number of sequences
my $minAltRds = 20; # minimum number of sequences with the alternative allele
my $maxRevOrient = 1; # maximum number of reverese orientation sequences for each allele
#####

my $in = shift(@ARGV);
open (IN, $in) or die "Could not read the infile = $in\n";
open (OUT, "> filtered_$in") or die "Could not write the outfile\n";

my $flag = 0;
my $cnt = 0;

while (<IN>){
	chomp;
	$flag = 1;
	if (m/^\#/){ ## header row, always write
		$flag = 1;
	}
	elsif (m/^scaffold/){ ## this is a sequence line, you migh need to edit this reg. expr.
		$flag = 1;
		if (m/[ACTGN]\,[ACTGN]/){ ## two alternative alleles identified
			$flag = 0;
		}
		m/DP=(\d+)/ or die "Syntax error, DP not found\n";
		if ($1 < $minCoverage){
			$flag = 0;
		}
		m/DP4=(\d+)\,(\d+)\,(\d+)\,(\d+)/ or die "Syntax error, DP4 not found\n";
		if (($3 < $minAltRds) && ($4 < $minAltRds)){
			$flag = 0;
		}
		elsif (($1 > $maxRevOrient && $2 > $maxRevOrient) || ($3 > $maxRevOrient && $4 > $maxRevOrient)){
			$flag = 0;
		}
		if ($flag == 1){
			$cnt++; ## this is a good SNV
		}
	}
	else{
		print "Warning, failed to match the chromosome or scaffold name regular expression for this line\n$_\n";
		$flag = 0;
	}
	if ($flag == 1){
		print OUT "$_\n";
	}
}
close (IN);
close (OUT);

print "Finished filtering $in\nRetained $cnt variable loci\n";
